/* lsystem3 -- simple lindenmayer system with audio realization of the
	graph using RTcmix WAVETABLE instrument (red lines == frequency)
	"myTimer.C" is the animator object (can't use RTtimeit because
	of graphics context problems)  -- BGG/2003
*/

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#define MAIN		/* So RTcmix globals will be right */
#include <globals.h>

#include "lsystem3.h"
#include <iostream.h>



IMPLEMENT_APP(lsystem3)

bool lsystem3::OnInit()
{
	wxFrame *frame = new wxFrame((wxFrame*) NULL, -1, "lsystem3",
		wxPoint(50, 50), wxSize(1000, 600),
		wxDEFAULT_FRAME_STYLE, "lsystem3");
	frame->Show(TRUE);
	SetTopWindow(frame);

	wxButton *button1 = new wxButton(frame, BUTTON_HIT1, "realize!",
		wxPoint(100, 550), wxSize(70, 20));

	wxButton *button2 = new wxButton(frame, BUTTON_HIT2, "go!",
		wxPoint(200, 550), wxSize(70, 20));

	wdc = new wxClientDC(frame);
	recurse_levels = 5;

	RTcmix *rrr = new RTcmix(44100.0, 2);
	sleep(1);
	rrr->cmd("load", 1, "WAVETABLE");
	rrr->cmd("makegen", 11, 1.0, 24.0, 1000.0, 0.0, 0.0, 1.0, 1.0, 9.0, 1.0, 10.0, 0.0);
	rrr->cmd("makegen", 5, 2.0, 10.0, 1000.0, 1.0);
	rrr->cmd("reset", 1, 44100.0);

	theTimer = new myTimer(wdc, rrr);

	return true;
}

void lsystem3::OnButton1(wxCommandEvent& command)
{
// realize the l-system
	int x, y, length;

	x = 0;
	y = 250;
	length = 1000/4;

	wdc->SetPen(*wxRED_PEN);
	wdc->DrawLine(x,y, length,y);

	recurse(x+length, y, length/2, recurse_levels-1);
}


void lsystem3::OnButton2(wxCommandEvent& command)
{
// start the audio interperetation, every 100 msecs
	theTimer->setX(0);
	theTimer->Start(100, wxTIMER_CONTINUOUS);
}

void lsystem3::recurse(int x, int y, int length, int rlevel)
{
	if (rlevel == 0) return;

	wdc->SetPen(*wxRED_PEN);
	wdc->DrawLine(x,y, x+length,y+20);
	wdc->DrawLine(x,y, x+length,y+30);
	wdc->DrawLine(x,y, x+length-10,y-10);

	recurse(x+length, y+20, length-rlevel, rlevel-1);
	recurse(x+length, y+30, length/2, rlevel-1);
	recurse(x+length-10, y-10, length/2, rlevel-1);
}

BEGIN_EVENT_TABLE(lsystem3, wxApp)
	EVT_BUTTON(BUTTON_HIT1, lsystem3::OnButton1)
	EVT_BUTTON(BUTTON_HIT2, lsystem3::OnButton2)
END_EVENT_TABLE()
